set timing on
set echo on

DROP TABLE tab1;
DROP TABLE tab2;

CREATE TABLE tab1 AS SELECT ROWNUM pk, 999999 skewed , a.* FROM dba_objects a, (select * from dual connect by level <=10) b;
CREATE TABLE tab2 AS SELECT ROWNUM pk, 999999 skewed , a.* FROM dba_objects a, (select * from dual connect by level <=20) b;

-- Only three rows with numbers other than '999999'
UPDATE tab1
   SET skewed = rownum
 WHERE ROWNUM < 4;
 
UPDATE tab2
   SET skewed = rownum
 WHERE ROWNUM < 4;
 

COMMIT ;

SELECT   skewed,
         COUNT (*) c
    FROM tab1
GROUP BY skewed
ORDER by skewed;

SELECT   skewed,
         COUNT (*) c
    FROM tab2
GROUP BY skewed
ORDER by skewed;


DROP INDEX tab1_idx1;
DROP INDEX tab1_idx2;
DROP INDEX tab2_idx1;
DROP INDEX tab2_idx2;

CREATE UNIQUE INDEX tab1_idx1 ON tab1(PK);

CREATE UNIQUE INDEX tab2_idx1 ON tab2(PK);





EXEC dbms_stats.delete_table_stats(user,'TAB1')
EXEC dbms_stats.delete_table_stats(user,'TAB2')
EXEC dbms_stats.gather_table_stats( user, 'TAB1', ESTIMATE_PERCENT=>100, method_opt=>'for all indexed columns',cascade=>true);
EXEC dbms_stats.gather_table_stats( user, 'TAB2', ESTIMATE_PERCENT=>100, method_opt=>'for all indexed columns',cascade=>true);


PAUSE 2 tables with Index on PK only.  Histogram stats on PK only 

ALTER SYSTEM FLUSH SHARED_POOL;
ALTER SESSION SET sql_trace=TRUE;

SELECT SUM (t1.pk * t1.skewed * t2.pk * t2.skewed)
  FROM tab1 t1,
       tab2 t2
 WHERE t1.pk = t2.pk
   AND t2.skewed = 2
/


ALTER SESSION SET sql_trace=FALSE
/


/* ************************************** Results ********************************

call     count       cpu    elapsed       disk      query    current        rows
------- ------  -------- ---------- ---------- ---------- ----------  ----------
Parse        1      0.20       0.22          0          0          0           0
Execute      1      0.00       0.00          0          0          0           0
Fetch        2      0.75       4.50      16840      23091          0           1
------- ------  -------- ---------- ---------- ---------- ----------  ----------
total        4      0.95       4.73      16840      23091          0           1

Rows     Row Source Operation
-------  ---------------------------------------------------
      1  SORT AGGREGATE (cr=23091 pr=16840 pw=0 time=4503990 us)
      1   HASH JOIN  (cr=23091 pr=16840 pw=0 time=4503954 us)
      1    TABLE ACCESS FULL TAB2 (cr=15392 pr=15094 pw=0 time=3823065 us)
 508030    TABLE ACCESS FULL TAB1 (cr=7699 pr=1746 pw=0 time=2032185 us)

********************************************************************************   */


PAUSE Lets look at the above Execution plan




-- Flush the shared pool
ALTER SYSTEM FLUSH SHARED_POOL;

-- Now gather histogram stats on non-index col, SKEWED
EXEC dbms_stats.gather_table_stats( user, 'TAB2', ESTIMATE_PERCENT=>100, method_opt=>'for columns SKEWED')


PAUSE  Now gather histogram stats on the SKEWED column and run the same SQL



alter session set SQL_TRACE=true;

SELECT SUM (t1.pk * t1.skewed * t2.pk * t2.skewed)
  FROM tab1 t1,
       tab2 t2
 WHERE t1.pk = t2.pk
   AND t2.skewed = 2
/

alter session set SQL_TRACE=false;   


/* ************************************** Results ********************************

call     count       cpu    elapsed       disk      query    current        rows
------- ------  -------- ---------- ---------- ---------- ----------  ----------
Parse        1      0.00       0.00          0          0          0           0
Execute      1      0.00       0.00          0          0          0           0
Fetch        2      0.45       3.85      15288      15396          0           1
------- ------  -------- ---------- ---------- ---------- ----------  ----------
total        4      0.45       3.86      15288      15396          0           1

Rows     Row Source Operation
-------  ---------------------------------------------------
      1  SORT AGGREGATE (cr=15396 pr=15288 pw=0 time=3852567 us)
      1   NESTED LOOPS  (cr=15396 pr=15288 pw=0 time=3852532 us)
      1    TABLE ACCESS FULL TAB2 (cr=15392 pr=15288 pw=0 time=3852465 us)
      1    TABLE ACCESS BY INDEX ROWID TAB1 (cr=4 pr=0 pw=0 time=43 us)
      1     INDEX UNIQUE SCAN TAB1_IDX1 (cr=3 pr=0 pw=0 time=20 us)(object id 57674)

